#include <wx/wx.h>
#include "GLCanvas.h"

BEGIN_EVENT_TABLE(GLCanvas, wxGLCanvas)
    EVT_SIZE(GLCanvas::OnSize)
    EVT_PAINT(GLCanvas::OnPaint)
	EVT_LEFT_DOWN(GLCanvas::OnLeftButtonDown)
	EVT_LEFT_UP(GLCanvas::OnLeftButtonUp)
	EVT_LEFT_DCLICK(GLCanvas::OnLeftButtonDoubleClick)
	EVT_MIDDLE_DOWN(GLCanvas::OnMiddleButtonDown)
	EVT_MIDDLE_UP(GLCanvas::OnMiddleButtonUp)
	EVT_MIDDLE_DCLICK(GLCanvas::OnMiddleButtonDoubleClick)
	EVT_RIGHT_DOWN(GLCanvas::OnRightButtonDown)
	EVT_RIGHT_UP(GLCanvas::OnRightButtonUp)
	EVT_RIGHT_DCLICK(GLCanvas::OnRightButtonDoubleClick)
	EVT_MOTION(GLCanvas::OnMouseMotion)
END_EVENT_TABLE()

GLCanvas::GLCanvas(wxWindow* parent, wxWindowID id, const wxPoint& pos, 
				   const wxSize& size, long style, const wxString& name)
	:wxGLCanvas(parent, id, pos, size, style, name)
{
}

GLCanvas::~GLCanvas()
{
}

void GLCanvas::SetTransformMatrix()
{
    int W, H;
    GetClientSize(&W, &H);
    glViewport(0, 0, W, H);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void GLCanvas::OnSize(wxSizeEvent& event)
{
    Refresh();
}

void GLCanvas::OnPaint(wxPaintEvent& event)
{
    wxPaintDC dc(this);
    SetCurrent();
    this->SetTransformMatrix();

    //begin draw 

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glColor3f(1.0f, 0.0f, 0.0f);
    glBegin(GL_LINES);
    glVertex2d(-0.9, -0.9);
    glVertex2d(0.9, 0.9);
    glEnd();

    glDisable(GL_BLEND);

	//end draw
    glFlush();
    SwapBuffers();
}

void GLCanvas::OnLeftButtonDown(wxMouseEvent &event)
{
    this->OnButtonDown(MOUSE_LEFT, event);
}

void GLCanvas::OnLeftButtonUp(wxMouseEvent &event)
{
    this->OnButtonUp(MOUSE_LEFT, event);
}

void GLCanvas::OnLeftButtonDoubleClick(wxMouseEvent &event)
{
    this->OnButtonDoubleClick(MOUSE_LEFT, event);
}

void GLCanvas::OnMiddleButtonDown(wxMouseEvent &event)
{
    this->OnButtonDown(MOUSE_MIDDLE, event);
}

void GLCanvas::OnMiddleButtonUp(wxMouseEvent &event)
{
    this->OnButtonUp(MOUSE_MIDDLE, event);
}

void GLCanvas::OnMiddleButtonDoubleClick(wxMouseEvent &event)
{
    this->OnButtonDoubleClick(MOUSE_MIDDLE, event);
}

void GLCanvas::OnRightButtonDown(wxMouseEvent &event)
{
    this->OnButtonDown(MOUSE_RIGHT, event);
}

void GLCanvas::OnRightButtonUp(wxMouseEvent &event)
{
    this->OnButtonUp(MOUSE_RIGHT, event);
}

void GLCanvas::OnRightButtonDoubleClick(wxMouseEvent &event)
{
    this->OnButtonDoubleClick(MOUSE_RIGHT, event);
}

void GLCanvas::OnMouseMotion(wxMouseEvent &event)
{
    SetCurrent();
    this->SetTransformMatrix();

    int key = this->GetKeyStatus(event);
	double x, y;
    this->MapScreenToData(event.GetX(), event.GetY(), x, y);
	//x y are within [-1, 1]. They are the position of the mouse in the data space. 
	//you can add your coding for processing the mouse event here
}

void GLCanvas::OnButtonDown(int mouse, wxMouseEvent &event)
{
    SetCurrent();
    this->SetTransformMatrix();

    int key = this->GetKeyStatus(event);
	double x, y;
    this->MapScreenToData(event.GetX(), event.GetY(), x, y);
	//x y are within [-1, 1]. They are the position of the mouse in the data space. 
	//you can add your coding for processing the mouse event here

}

void GLCanvas::OnButtonUp(int mouse, wxMouseEvent &event)
{
    SetCurrent();
    this->SetTransformMatrix();

    int key = this->GetKeyStatus(event);
	double x, y;
    this->MapScreenToData(event.GetX(), event.GetY(), x, y);
	//x y are within [-1, 1]. They are the position of the mouse in the data space. 
	//you can add your coding for processing the mouse event here
}

void GLCanvas::OnButtonDoubleClick(int mouse, wxMouseEvent &event)
{
    SetCurrent();
    this->SetTransformMatrix();

    int key = this->GetKeyStatus(event);
	double x, y;
	//x y are within [-1, 1]. They are the position of the mouse in the data space. 
	//you can add your coding for processing the mouse event here
    this->MapScreenToData(event.GetX(), event.GetY(), x, y);
}

void GLCanvas::MapScreenToData(int sx, int sy, double &dx, double &dy)
{
    GLdouble modelMatrix[16], projMatrix[16];
    GLint viewport[4];
    GLdouble x, y, z;

    glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);
    glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
    glGetIntegerv(GL_VIEWPORT, viewport);
    int W, H;
    GetClientSize(&W, &H);
    gluUnProject(sx, H-1-sy, 0., modelMatrix, projMatrix, viewport, &x, &y, &z);
    
	dx = x;
	dy = y;
}

int GLCanvas::GetKeyStatus(const wxMouseEvent &event)
{
    int status = 0;
    if (event.ShiftDown())
        status |= KEY_SHIFT;
    if (event.AltDown())
        status |= KEY_ALT;
    if (event.ControlDown())
        status |= KEY_CTRL;
    return status;
}

